@@ -48,19 +48,6 @@ typedef struct _EMapiConnection EMapiConnection;
 typedef struct _EMapiConnectionClass EMapiConnectionClass;
 typedef struct _EMapiConnectionPrivate EMapiConnectionPrivate;
 
-typedef enum {
-	MAPI_OPTIONS_FETCH_ATTACHMENTS = 1<<0,
-	MAPI_OPTIONS_FETCH_RECIPIENTS = 1<<1,
-	MAPI_OPTIONS_FETCH_BODY_STREAM = 1<<2,
-	MAPI_OPTIONS_FETCH_GENERIC_STREAMS = 1<<3,
-	MAPI_OPTIONS_FETCH_GAL = 1 <<4,
-	MAPI_OPTIONS_DONT_SUBMIT = 1<<5,
-	MAPI_OPTIONS_GETBESTBODY = 1<<6,
-	MAPI_OPTIONS_USE_PFSTORE = 1<<7,
-	MAPI_OPTIONS_DONT_OPEN_MESSAGE = 1<<8,
-	MAPI_OPTIONS_DELETE_ON_SUBMIT_FAILURE = 1<<9
-} ExchangeMAPIOptions;
-
 typedef struct {
 	uint32_t pidlid_propid; /* PidLid or PidName legacy property named ID to resolve */
 	uint32_t propid;	/* resolved prop ID; equals to MAPI_E_RESERVED when not found or other error */
@@ -74,24 +61,27 @@ typedef struct {
 	time_t last_modified;	/* PidTagLastModificationTime as UTC */
 } ListObjectsData;
 
-struct _EMapiObject;
-struct _EMapiRecipient;
-struct _EMapiAttachment;
+typedef struct _EMapiStreamedProp {
+	uint32_t proptag;
+	uint64_t cb;
+	const uint8_t *lpb; /* taken from the original mapi prop, no need to copy the memory */
+} EMapiStreamedProp;
 
-typedef struct _EMapiObject EMapiObject;
 typedef struct _EMapiRecipient EMapiRecipient;
 typedef struct _EMapiAttachment EMapiAttachment;
+typedef struct _EMapiObject EMapiObject;
 
-struct _EMapiRecipient
-{
+struct _EMapiRecipient {
 	struct mapi_SPropValue_array properties;
 
 	EMapiRecipient *next;
 };
 
-struct _EMapiAttachment
-{
+struct _EMapiAttachment {
 	struct mapi_SPropValue_array properties;
+	EMapiStreamedProp *streamed_properties; /* use get/add functions for these */
+	guint32 streamed_properties_count;
+
 	EMapiObject *embedded_object;
 
 	EMapiAttachment *next;
@@ -99,6 +89,9 @@ struct _EMapiAttachment
 
 struct _EMapiObject {
 	struct mapi_SPropValue_array properties;
+	EMapiStreamedProp *streamed_properties; /* use get/add functions for these */
+	guint32 streamed_properties_count;
+
 	EMapiRecipient *recipients; /* NULL when none */
 	EMapiAttachment *attachments; /* NULL when none */
 
@@ -110,6 +103,16 @@ void			e_mapi_recipient_free		(EMapiRecipient *recipient);
 
 EMapiAttachment *	e_mapi_attachment_new		(TALLOC_CTX *mem_ctx);
 void			e_mapi_attachment_free		(EMapiAttachment *attachment);
+void			e_mapi_attachment_add_streamed	(EMapiAttachment *attachment,
+							 uint32_t proptag,
+							 uint64_t cb,
+							 const uint8_t *lpb); /* this might be created inside the attachment TALLOC_CTX */
+EMapiStreamedProp *	e_mapi_attachment_get_streamed	(EMapiAttachment *attachment,
+							 uint32_t proptag);
+gboolean		e_mapi_attachment_get_bin_prop	(EMapiAttachment *attachment,
+							 uint32_t proptag,
+							 uint64_t *cb,
+							 const uint8_t **lpb);
 
 EMapiObject *		e_mapi_object_new		(TALLOC_CTX *mem_ctx);
 void			e_mapi_object_free		(EMapiObject *object);
@@ -117,6 +120,48 @@ void			e_mapi_object_add_recipient	(EMapiObject *object,
 							 EMapiRecipient *recipient);
 void			e_mapi_object_add_attachment	(EMapiObject *object,
 							 EMapiAttachment *attachment);
+void			e_mapi_object_add_streamed	(EMapiObject *object,
+							 uint32_t proptag,
+							 uint64_t cb,
+							 const uint8_t *lpb); /* this might be created inside the attachment TALLOC_CTX */
+EMapiStreamedProp *	e_mapi_object_get_streamed	(EMapiObject *object,
+							 uint32_t proptag);
+gboolean		e_mapi_object_get_bin_prop	(EMapiObject *object,
+							 uint32_t proptag,
+							 uint64_t *cb,
+							 const uint8_t **lpb);
+
+#define E_MAPI_PERMISSION_MEMBER_ID_ANONYMOUS_CLIENT	(~((uint64_t) 0))
+#define E_MAPI_PERMISSION_MEMBER_ID_DEFAULT_USER	((uint64_t) 0)
+
+typedef enum {
+	E_MAPI_PERMISSION_BIT_FREE_BUSY_DETAILED	= 0x00001000,
+	E_MAPI_PERMISSION_BIT_FREE_BUSY_SIMPLE		= 0x00000800,
+	E_MAPI_PERMISSION_BIT_FOLDER_VISIBLE		= 0x00000400,
+	E_MAPI_PERMISSION_BIT_FOLDER_CONTACT		= 0x00000200,
+	E_MAPI_PERMISSION_BIT_FOLDER_OWNER		= 0x00000100,
+	E_MAPI_PERMISSION_BIT_CREATE_SUBFOLDER		= 0x00000080,
+	E_MAPI_PERMISSION_BIT_DELETE_ANY		= 0x00000040,
+	E_MAPI_PERMISSION_BIT_EDIT_ANY			= 0x00000020,
+	E_MAPI_PERMISSION_BIT_DELETE_OWNED		= 0x00000010,
+	E_MAPI_PERMISSION_BIT_EDIT_OWNED		= 0x00000008,
+	E_MAPI_PERMISSION_BIT_CREATE			= 0x00000002,
+	E_MAPI_PERMISSION_BIT_READ_ANY			= 0x00000001
+} EMapiPermissionBits;
+
+typedef struct {
+	gchar *username;		/* PidTagMemberName - display name for a user */
+
+	struct SBinary_short entry_id;	/* PidTagEntryId of the user */
+	uint64_t member_id;		/* PidTagMemberId of the user */
+	uint32_t member_rights;		/* PidTagMemberRights of the user */
+} EMapiPermissionEntry;
+
+EMapiPermissionEntry *	e_mapi_permission_entry_new	(const gchar *username,
+							 const struct SBinary_short *entry_id,
+							 uint64_t member_id,
+							 uint32_t member_rights);
+void			e_mapi_permission_entry_free	(EMapiPermissionEntry *entry);
 
 typedef enum {
 	E_MAPI_CREATE_FLAG_NONE		= 0,
@@ -158,7 +203,7 @@ typedef gboolean (*WriteObjectCB)		(EMapiConnection *conn,
 						 gpointer user_data,
 						 GCancellable *cancellable,
 						 GError **perror);
-typedef gboolean (*GetFolderPropertiesCB)	(EMapiConnection *conn,
+typedef gboolean (*GetPropertiesCB)		(EMapiConnection *conn,
 						 TALLOC_CTX *mem_ctx,
 						 /* const */ struct mapi_SPropValue_array *properties,
 						 gpointer user_data,
@@ -196,6 +241,28 @@ gboolean		e_mapi_connection_reconnect		(EMapiConnection *conn,
 gboolean		e_mapi_connection_close			(EMapiConnection *conn);
 gboolean		e_mapi_connection_connected		(EMapiConnection *conn);
 
+gboolean		e_mapi_connection_test_foreign_folder	(EMapiConnection *conn,
+								 const gchar *username,
+								 const gchar *folder_name,
+								 mapi_id_t *fid, /* out */
+								 GCancellable *cancellable,
+								 GError **perror);
+
+gboolean		e_mapi_connection_peek_store		(EMapiConnection *conn,
+								 gboolean public_store,
+								 const gchar *foreign_username,
+								 mapi_object_t **obj_store, /* out */
+								 GCancellable *cancellable,
+								 GError **perror);
+
+gboolean		e_mapi_connection_get_store_quotas	(EMapiConnection *conn,
+								 mapi_object_t *obj_store, /* can be NULL */
+								 uint64_t *current_size, /* out */
+								 uint64_t *receive_quota, /* out */
+								 uint64_t *send_quota, /* out */
+								 GCancellable *cancellable,
+								 GError **perror);
+
 gboolean		e_mapi_connection_open_default_folder	(EMapiConnection *conn,
 								 uint32_t olFolderIdentifier,
 								 mapi_object_t *obj_folder, /* out */
@@ -214,6 +281,13 @@ gboolean		e_mapi_connection_open_public_folder	(EMapiConnection *conn,
 								 GCancellable *cancellable,
 								 GError **perror);
 
+gboolean		e_mapi_connection_open_foreign_folder	(EMapiConnection *conn,
+								 const gchar *username,
+								 mapi_id_t fid,
+								 mapi_object_t *obj_folder, /* out */
+								 GCancellable *cancellable,
+								 GError **perror);
+
 gboolean		e_mapi_connection_close_folder		(EMapiConnection *conn,
 								 mapi_object_t *obj_folder,
 								 GCancellable *cancellable,
@@ -223,11 +297,25 @@ gboolean		e_mapi_connection_get_folder_properties	(EMapiConnection *conn,
 								 mapi_object_t *obj_folder,
 								 BuildReadPropsCB brp_cb,
 								 gpointer brp_cb_user_data,
-								 GetFolderPropertiesCB cb,
+								 GetPropertiesCB cb,
 								 gpointer cb_user_data,
 								 GCancellable *cancellable,
 								 GError **perror);
 
+gboolean		e_mapi_connection_get_permissions	(EMapiConnection *conn,
+								 mapi_object_t *obj_folder,
+								 gboolean with_freebusy,
+								 GSList **entries, /* out, EMapiPermissionEntry */
+								 GCancellable *cancellable,
+								 GError **perror);
+
+gboolean		e_mapi_connection_set_permissions	(EMapiConnection *conn,
+								 mapi_object_t *obj_folder,
+								 gboolean with_freebusy,
+								 const GSList *entries, /* EMapiPermissionEntry */
+								 GCancellable *cancellable,
+								 GError **perror);
+
 gboolean		e_mapi_connection_list_objects		(EMapiConnection *conn,
 								 mapi_object_t *obj_folder,
 								 BuildRestrictionsCB build_rs_cb,
@@ -293,6 +381,8 @@ gboolean		e_mapi_connection_list_gal_objects	(EMapiConnection *conn,
 
 gboolean		e_mapi_connection_transfer_gal_objects	(EMapiConnection *conn,
 								 const GSList *mids,
+								 BuildReadPropsCB brp_cb, /* NULL for all supported */
+								 gpointer brp_cb_user_data,
 								 TransferObjectCB cb,
 								 gpointer cb_user_data,
 								 GCancellable *cancellable,
@@ -306,39 +396,37 @@ gboolean		e_mapi_connection_transfer_gal_object	(EMapiConnection *conn,
 								 GError **perror);
 
 gboolean		e_mapi_connection_get_public_folder	(EMapiConnection *conn,
-								 mapi_object_t *obj_object,
+								 mapi_object_t *obj_folder, /* out */
 								 GCancellable *cancellable,
 								 GError **perror);
 
-mapi_id_t		e_mapi_connection_create_folder		(EMapiConnection *conn,
-								 uint32_t olFolder,
-								 mapi_id_t pfid,
-								 guint32 fid_options,
+gboolean		e_mapi_connection_create_folder		(EMapiConnection *conn,
+								 mapi_object_t *obj_parent_folder, /* in */
 								 const gchar *name,
+								 const gchar *new_folder_type, /* usually IPF_NOTE and similar */
+								 mapi_id_t *new_fid, /* out */
 								 GCancellable *cancellable,
 								 GError **perror);
+
 gboolean		e_mapi_connection_remove_folder		(EMapiConnection *conn,
-								 mapi_id_t fid,
-								 guint32 fid_options,
+								 mapi_object_t *obj_store, /* in, store, to which folder belongs */
+								 mapi_id_t fid_to_remove,
 								 GCancellable *cancellable,
 								 GError **perror);
+
 gboolean		e_mapi_connection_empty_folder		(EMapiConnection *conn,
-								 mapi_id_t fid,
-								 guint32 fid_options,
+								 mapi_object_t *obj_folder,
 								 GCancellable *cancellable,
 								 GError **perror);
 gboolean		e_mapi_connection_rename_folder		(EMapiConnection *conn,
-								 mapi_id_t fid,
-								 guint32 fid_options,
+								 mapi_object_t *obj_folder,
 								 const gchar *new_name,
 								 GCancellable *cancellable,
 								 GError **perror);
 gboolean		e_mapi_connection_move_folder		(EMapiConnection *conn,
-								 mapi_id_t src_fid,
-								 mapi_id_t src_parent_fid,
-								 guint32 src_fid_options,
-								 mapi_id_t des_fid,
-								 guint32 des_fid_options,
+								 mapi_object_t *src_obj_folder,
+								 mapi_object_t *src_parent_obj_folder,
+								 mapi_object_t *des_obj_folder,
 								 const gchar *new_name,
 								 GCancellable *cancellable,
 								 GError **perror);
@@ -348,33 +436,20 @@ mapi_id_t		e_mapi_connection_get_default_folder_id	(EMapiConnection *conn,
 								 GError **perror);
 
 gboolean		e_mapi_connection_set_flags		(EMapiConnection *conn,
-								 uint32_t olFolder,
-								 mapi_id_t fid,
-								 guint32 fid_options,
+								 mapi_object_t *obj_folder,
 								 GSList *mid_list,
 								 uint32_t flag,
 								 GCancellable *cancellable,
 								 GError **perror);
 gboolean		e_mapi_connection_remove_items		(EMapiConnection *conn,
-								 uint32_t olFolder,
-								 mapi_id_t fid,
-								 guint32 fid_options,
+								 mapi_object_t *obj_folder,
 								 const GSList *mids, /* data is (mapi_id_t *) */
 								 GCancellable *cancellable,
 								 GError **perror);
-gboolean		e_mapi_connection_copy_items		(EMapiConnection *conn,
-								 mapi_id_t src_fid,
-								 guint32 src_fid_options,
-								 mapi_id_t dest_fid,
-								 guint32 dest_fid_options,
-								 GSList *mids,
-								 GCancellable *cancellable,
-								 GError **perror);
-gboolean		e_mapi_connection_move_items		(EMapiConnection *conn,
-								 mapi_id_t src_fid,
-								 guint32 src_fid_options,
-								 mapi_id_t dest_fid,
-								 guint32 dest_fid_options,
+gboolean		e_mapi_connection_copymove_items	(EMapiConnection *conn,
+								 mapi_object_t *src_obj_folder,
+								 mapi_object_t *des_obj_folder,
+								 gboolean do_copy,
 								 GSList *mids,
 								 GCancellable *cancellable,
 								 GError **perror);
@@ -393,13 +468,13 @@ gboolean		e_mapi_connection_get_pf_folders_list	(EMapiConnection *conn,
 GSList *		e_mapi_connection_peek_folders_list	(EMapiConnection *conn);
 
 gboolean		e_mapi_connection_resolve_named_props	(EMapiConnection *conn,
-								 mapi_id_t fid,
+								 mapi_object_t *obj_folder,
 								 EResolveNamedIDsData *named_ids_list,
 								 guint named_ids_n_elems,
 								 GCancellable *cancellable,
 								 GError **perror);
 uint32_t		e_mapi_connection_resolve_named_prop	(EMapiConnection *conn,
-								 mapi_id_t fid,
+								 mapi_object_t *obj_folder,
 								 uint32_t pidlid_propid,
 								 GCancellable *cancellable,
 								 GError **perror);
@@ -414,6 +489,15 @@ gchar *			e_mapi_connection_ex_to_smtp		(EMapiConnection *conn,
 								 GCancellable *cancellable,
 								 GError **perror);
 
+gboolean		e_mapi_connection_resolve_username	(EMapiConnection *conn,
+								 const gchar *to_resolve,
+								 BuildReadPropsCB brp_cb,
+								 gpointer brp_cb_user_data,
+								 GetPropertiesCB cb,
+								 gpointer cb_user_data,
+								 GCancellable *cancellable,
+								 GError **perror);
+
 /* Push notifications APIs */
 
 gboolean		e_mapi_connection_enable_notifications	(EMapiConnection *conn,
